<?php

namespace Drupal\diff_translation\Controller;

use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Entity\TranslatableInterface;
use Drupal\Core\Routing\RouteMatchInterface;
use Drupal\diff\Controller\PluginRevisionController;
use Symfony\Component\HttpKernel\Exception\AccessDeniedHttpException;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;

/**
 * Provide a controller to see diff between last revisions of different content.
 */
class DiffTranslationController extends PluginRevisionController
{
  public function compareEntityTranslation(
    RouteMatchInterface $route_match,
    string $left_lang,
    string $right_lang,
  ) {

    $entity_type_id = array_key_first($route_match->getParameters()->all());
    /** @var \Drupal\Core\Entity\EntityInterface $entity */
    $entity = $route_match->getParameter($entity_type_id);

    if (!$entity instanceof TranslatableInterface
      || !$entity->hasTranslation($left_lang)
      || !$entity->hasTranslation($right_lang)
    ) {
      throw new NotFoundHttpException();
    }

    $left_entity = $entity->getTranslation($left_lang);
    $right_entity = $entity->getTranslation($right_lang);

    if (!$left_entity->access('view') || !$right_entity->access('view')) {
      throw new AccessDeniedHttpException();
    }

    $build = [
      '#title' => $this->t('Translation differences to %title', ['%title' => $entity->label()]),
    ];

    // Perform comparison only if both entity revisions loaded successfully.
    if ($plugin = $this->diffLayoutManager->createInstance('split_fields_diff_translation')) {
      $build = array_merge_recursive($build, $plugin->build($left_entity, $right_entity, $entity));
      $build['diff']['#prefix'] = '<div class="diff-responsive-table-wrapper">';
      $build['diff']['#suffix'] = '</div>';
      $build['diff']['#attributes']['class'][] = 'diff-responsive-table';
    }

    $build['#attached']['library'][] = 'diff/diff.general';
    return $build;
  }
}
