<?php

namespace Drupal\diff_translation\Hook;

use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\Core\Hook\Attribute\Hook;
use Drupal\Core\Routing\RouteMatchInterface;
use Drupal\Core\Url;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Hook implementations for diff_translation.
 */
class DiffTranslationHooks implements ContainerInjectionInterface {

  /**
   * Construct the diff translation hooks.
   *
   * @param \Drupal\Core\Routing\RouteMatchInterface $match
   *   The current route matcher.
   */
  public function __construct(
    protected readonly RouteMatchInterface $match
  ) {
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container): self {
    return new self(
      $container->get('current_route_match')
    );
  }

  /**
   * @param mixed[] $data
   *   Menu local tasks to alter.
   * @param string $route_name
   *   Current route name.
   *
   * @return void
   */
  #[Hook('menu_local_tasks_alter')]
  public function menuLocalTasksAlter(array &$data, string $route_name): void {
    if (isset($data['tabs'][0])) {
      $route_parameters = $this->match->getParameters();
      foreach ($route_parameters as $route_parameter) {
        if ($route_parameter instanceof ContentEntityInterface) {
          $entity_type = $route_parameter->getEntityTypeId();
          $task_id = 'diff_translation.local_tasks:diff_translation.' . $entity_type;
          if (isset($data['tabs'][0][$task_id]['#link']['url'])) {
            $languages = $route_parameter->getTranslationLanguages();
            if (count($route_parameter->getTranslationLanguages()) < 2) {
              unset($data['tabs'][0][$task_id]);
              return;
            }

            $other_languages = $route_parameter->getTranslationLanguages(FALSE);
            $langs = array_keys($languages);
            $other_langs = array_keys($other_languages);
            $original_lang = array_diff($langs, $other_langs)[0];

            $original_entity = $route_parameter->getTranslation($original_lang);
            $compare_entity = $route_parameter->getTranslation($other_langs[0]);
            if (!$original_entity->access('view') || !$compare_entity->access('view')) {
              unset($data['tabs'][0][$task_id]);
              return;
            }

            $data['tabs'][0][$task_id]['#link']['url'] = new Url(
              'entity.' . $entity_type . '.diff_translation',
              [
                $entity_type => $route_parameter->id(),
                'left_lang' => $original_lang,
                'right_lang' => $other_langs[0],
              ]
            );

          }
        }
      }
    }
  }
}
